using System;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{
		
		public class Console : IConsole, ISerializable
		{
					
			public ICartridge Cartridge
			{
				get
				{
					return CartridgeProperty;
				}
				private set
				{
					CartridgeProperty = value;
				}
			}

			private ICartridge CartridgeProperty = null;

			public IController[] ControllerArray
            {
            	get
            	{
            		return new IController[] { (IController)P1 };
            	}
            }
			//private Register.IE IE = null;

			private Register.IF IF = null;

			//private readonly InputDevice InputDevice = null;
			public IInterface Interface
			{
				get
				{
					return InterfaceProperty;
				}
				private set
				{
					InterfaceProperty = value;
				}
			}
			
			private IInterface InterfaceProperty = null;
			
			public int InterruptMask
			{
				get
				{
					return InterruptMaskProperty;
				}
				set
				{
					InterruptMaskProperty = value;
				}
			}
			
			private int InterruptMaskProperty = 1; 

			public IMemory Memory
			{
				get
				{
					return MemoryProperty;
				}
				private set
				{
					MemoryProperty = value;
				}
			}
			
			private IMemory MemoryProperty = null;
			
			private Register.P1 P1 = null;

			public IProcessor Processor
			{
				get
				{
					return ProcessorProperty;
				}
				private set
				{
					ProcessorProperty = value;
				}
			}

			private IProcessor ProcessorProperty = null;

            private readonly IRegister[] RegisterArray = new IRegister[0x100];

            public ISerializable[] SerializationArray
            {
            	get
            	{
            		return new ISerializable[]
            		{ 
            			(ISerializable)this,
            			(ISerializable)Processor,
            			(ISerializable)Memory,
            			//(ISerializable)VideoProcessor,
            			(ISerializable)VideoMemory,
            		};
            	}
            }
            
            private double Time = 0.0;
            
			private Register.TMR TMR = null;
				
            public IVideoMemory VideoMemory
			{
				get
				{
					return VideoMemoryProperty;
				}
				private set
				{
					VideoMemoryProperty = value;
				}
			}
			
			private IVideoMemory VideoMemoryProperty = null;

			public IVideoProcessor VideoProcessor
			{
				get
				{
					return VideoProcessorProperty;
				}
				private set
				{
					VideoProcessorProperty = value;
				}
			}
			
			private IVideoProcessor VideoProcessorProperty = null;

			public Console( IInterface Interface )
			{
				this.Interface = Interface;
				return;
			}
			
			private void CreateRegisters( bool ColorGameBoy )
			{
				RegisterArray[0x00] = P1 = new Register.P1( this );
				RegisterArray[0x04] = TMR = new Register.TMR( this );
				RegisterArray[0x05] = RegisterArray[0x04];
				RegisterArray[0x06] = RegisterArray[0x04];
				RegisterArray[0x07] = RegisterArray[0x04];
				RegisterArray[0x0F] = IF = new Register.IF( this );
				RegisterArray[0xFF] = new Register.IE( this );
				/*RegisterArray[0x10] = new Register.SC1();
				RegisterArray[0x11] = RegisterArray[0x10];
				RegisterArray[0x12] = RegisterArray[0x10];
				RegisterArray[0x13] = RegisterArray[0x10];
				RegisterArray[0x14] = RegisterArray[0x10];				
				RegisterArray[0x16] = new Register.SC2();
				RegisterArray[0x17] = RegisterArray[0x16];
				RegisterArray[0x18] = RegisterArray[0x16];
				RegisterArray[0x19] = RegisterArray[0x16];				
				RegisterArray[0x1A] = new Register.SC3();
				RegisterArray[0x1B] = RegisterArray[0x1A];
				RegisterArray[0x1C] = RegisterArray[0x1A];
				RegisterArray[0x1D] = RegisterArray[0x1A];
				RegisterArray[0x1E] = RegisterArray[0x1A];
				RegisterArray[0x20] = new Register.SC4();
				RegisterArray[0x21] = RegisterArray[0x20];
				RegisterArray[0x22] = RegisterArray[0x20];
				RegisterArray[0x23] = RegisterArray[0x20];
				RegisterArray[0x24] = new Register.SCTRL();
				RegisterArray[0x25] = RegisterArray[0x24];
				RegisterArray[0x26] = RegisterArray[0x24];*/
				RegisterArray[0x40] = new Register.LCDC( this );
				RegisterArray[0x41] = RegisterArray[0x40];
				RegisterArray[0x44] = RegisterArray[0x40];
				RegisterArray[0x45] = RegisterArray[0x40];
				RegisterArray[0x42] = new Register.SCR();
				RegisterArray[0x43] = RegisterArray[0x42];
				RegisterArray[0x46] = new Register.DMA( this );
				RegisterArray[0x47] = new Register.BGP();
				RegisterArray[0x48] = new Register.OBP0();
				RegisterArray[0x49] = new Register.OBP1();
				RegisterArray[0x4A] = new Register.WIN();
				RegisterArray[0x4B] = RegisterArray[0x4A];
				if( ColorGameBoy )
				{
					RegisterArray[0x4D] = new Register.KEY1();
					RegisterArray[0x4F] = new Register.BNK();
					RegisterArray[0x70] = RegisterArray[0x4F];
					RegisterArray[0x51] = new Register.HDMA( this );
					RegisterArray[0x52] = RegisterArray[0x51];
					RegisterArray[0x53] = RegisterArray[0x51];
					RegisterArray[0x54] = RegisterArray[0x51];
					RegisterArray[0x55] = RegisterArray[0x51];
					RegisterArray[0x68] = new Register.BCP();
					RegisterArray[0x69] = RegisterArray[0x68];
					RegisterArray[0x6A] = new Register.OCP();
					RegisterArray[0x6B] = RegisterArray[0x6A];
				}
				for( int i = 0; i < RegisterArray.Length; i++ )
				{
					if( RegisterArray[i] == null )
					{
						RegisterArray[i] = new Register();
					}
				}
				return;
			}
			
			/*public void GenerateInterrupt( int Interrupt )
			/
				/*if( Interrupt >= Interrupts.TimerOverflow )
				{
					return;
				}*/
				/*if( ( IE.Value & Interrupt ) != 0 )
				{
					IF.Value = (byte)( IF.Value | Interrupt );
				}
				//Processor.RequestInterrupt( Interrupt );
				//return;
			}*/
			
			public IRegister GetRegister( int Address )
			{
				if( Address >= 0xFF00 )
				{
                	return RegisterArray[Address&0xFF];
				}
				return null;
			}

			public T GetRegister<T>() where T : IRegister
			{
				for( int i = 0; i < RegisterArray.Length; i++ )
				{
					if( RegisterArray[i] is T )
					{
						return (T)RegisterArray[i];
					}
				}
				return default( T );
			}
            
            public void Load( IInputStream InputStream )
            {
            	Cartridge NewCartridge = new Cartridge( Interface, InputStream );
            	if( NewCartridge.ROM != null )
            	{
            		this.Cartridge = NewCartridge;
            		this.Reset();
            	}
            	return;
            }
            
            public void Reset()
            {
            	this.CreateRegisters( ((Cartridge)Cartridge).ColorGameBoy );
            	this.VideoMemory = new VideoMemory( this );
				this.Memory = ((Cartridge)Cartridge).GetMapper( this );
            	this.Processor = new Processor( this );
				this.VideoProcessor = new VideoProcessor( this );
				//Processor.Reset();
            	//VideoProcessor.Reset();
            	this.Interface.Timer.Reset();
            	Time = 0.0;
            	return;
            }

			public void Update()
			{
				double TimeTaken = 0.0;
				Time += Interface.Timer.GetInterval( this );
				TMR.Update();
				while( Time > 0.0 )
				{
					TimeTaken = VideoProcessor.Run( Time );
					Processor.Run( TimeTaken );
					Time -= TimeTaken;
				}
				return;
			}
			
		}

	}

}
