﻿using System;
using System.Runtime.InteropServices;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{
		
		public partial class Processor : IProcessor
		{
			
			[StructLayout(LayoutKind.Explicit)]
			private struct RegisterPair
			{
						
				[FieldOffset(0)]
				public byte LowerByte;
						
				[FieldOffset(1)]
				public byte UpperByte;
						
				[FieldOffset(0)]
				public ushort Value;
				
				public RegisterPair( ushort Value )
				{
					this.LowerByte = 0;
					this.UpperByte = 0;
					this.Value = Value;
					return;
				}

			/*	public static implicit operator RegisterPair( ushort Value )
				{
					return new RegisterPair( Value );
				}*/

			}
			
			private byte A
			{
				get
				{
					return RegisterPairArray[4].UpperByte;
				}
				set
				{
					RegisterPairArray[4].UpperByte = value;
				}
			}
			
			private ushort AF
			{
				get
				{
					return RegisterPairArray[4].Value;
				}
				set
				{
					RegisterPairArray[4].Value = value;
				}
			}
			
			private ushort BC
			{
				get
				{
					return RegisterPairArray[0].Value;
				}
				set
				{
					RegisterPairArray[0].Value = value;
				}
			}
			
			private byte C
			{
				get
				{
					return RegisterPairArray[0].LowerByte;
				}
				set
				{
					RegisterPairArray[0].LowerByte = value;
				}
			}
			
			private ushort DE
			{
				get
				{
					return RegisterPairArray[1].Value;
				}
				set
				{
					RegisterPairArray[1].Value = value;
				}
			}
			
			private byte F
			{
				get
				{
					return RegisterPairArray[4].LowerByte;
				}
				set
				{
					RegisterPairArray[4].LowerByte = value;
				}
			}

			private ushort HL
			{
				get
				{
					return RegisterPairArray[2].Value;
				}
				set
				{
					RegisterPairArray[2].Value = value;
				}
			}

			private ushort IFF = 0;
			
			//private bool IME = false;

			private ushort PC = 0x0100;
	
			private ushort SP
			{
				get
				{
					return RegisterPairArray[3].Value;
				}
				set
				{
					RegisterPairArray[3].Value = value;
				}
			}

			protected readonly string[] RegisterNameArray = new string[] { "B", "C", "D", "E", "H", "L", "(HL)", "A" };
			
			private readonly RegisterPair[] RegisterPairArray = new RegisterPair[]
			{
				new RegisterPair( 0x0013 ), // BC
				new RegisterPair( 0x00D8 ), // DE
				new RegisterPair( 0x014D ), // HL
				new RegisterPair( 0xFFFE ), // SP
				new RegisterPair( 0x01B0 ), // AF
			};
				
			protected readonly string[] RegisterPairNameArray = new string[] { "BC", "DE", "HL", "SP", "AF" };
					
			protected byte GetRegisterValue( int Index )
			{
				if( Index != 6 )
				{
					if( Index == 7 )
					{
						return A;
					}
					else if( ( Index & 0x01 ) == 0 )
					{
						return RegisterPairArray[Index>>1].UpperByte;
					}
					else
					{
						return RegisterPairArray[Index>>1].LowerByte;
					}
				}
				return Memory.ReadU8( HL );
			}
			
			protected byte GetRegisterValue( int Index, bool UsePC )
			{
				if( Index != 6 )
				{
					if( Index == 7 )
					{
						return A;
					}
					else if( ( Index & 0x01 ) == 0 )
					{
						return RegisterPairArray[Index>>1].UpperByte;
					}
					else
					{
						return RegisterPairArray[Index>>1].LowerByte;
					}
				}
				return ( UsePC ? Memory.ReadU8( PC++ ) : Memory.ReadU8( HL ) );
			}
			
			protected void SetRegisterValue( int Index, byte Value )
			{
				if( Index != 6 )
				{
					if( Index == 7 )
					{
						A = Value;
					}
					else if( ( Index & 0x01 ) == 0 )
					{
						RegisterPairArray[Index>>1].UpperByte = Value;
					}
					else
					{
						RegisterPairArray[Index>>1].LowerByte = Value;
					}
				}
				else
				{
					Memory.WriteU8( HL, Value );
				}
				return;
			}

		}
		
	}
	
}
			
			
