﻿using System;
using GameKnight.Core;

namespace GameKnight
{
	
	namespace GameBoy
	{

		public partial class Register : IRegister
		{

			public class BCP : IRegister, IPalette
			{

				public uint[] ColorArray
				{
					get
					{
						return ColorArrayProperty;
					}
				}
				
				private readonly uint[] ColorArrayProperty = new uint[32];

				private readonly byte[] DataArray = new byte[2] { 0xC0, 0xC0 };

				public int Entries
				{
					get
					{
						return ColorArray.Length;
					}
				}

                private bool HighByte
				{
					get
					{
						return ( ( DataArray[0] & 0x01 ) != 0 );
					}
				}

                private readonly ushort[] PaletteArray = new ushort[32];

				private int PaletteIndex
				{
					get
					{
						return ( ( ( DataArray[0] >> 1 ) & 0x1F ) );
					}
				}

				public BCP()
				{
					return;
				}

				public byte this[int Address]
				{
                	get
                	{
						return DataArray[Address&0x01];
                	}
                	set
                	{
                		DataArray[Address&0x01] = value;
						switch( Address & 0x01 )
						{
							case 0: // BCPS
								if( HighByte )
								{
									DataArray[1] = (byte)( PaletteArray[PaletteIndex] >> 8 );
								}
								else
								{
									DataArray[1] = (byte)PaletteArray[PaletteIndex];
								}
								break;
							case 1: // BCPD
								if( HighByte )
								{
									PaletteArray[PaletteIndex] = (ushort)( ( ( value & 0x7F ) << 8 ) | ( PaletteArray[PaletteIndex] & 0xFF ) );
								}
								else
								{
									PaletteArray[PaletteIndex] = (ushort)( ( PaletteArray[PaletteIndex] & 0x7F00 ) | value );
								}
								ColorArray[PaletteIndex] = (uint)( ( ( PaletteArray[PaletteIndex] & 0x001F ) << 19 ) | ( ( PaletteArray[PaletteIndex] & 0x03E0 ) << 6 ) | ( ( PaletteArray[PaletteIndex] & 0x7C00 ) >> 7 ) );
								if( ( DataArray[0] & 0x80 ) != 0 )
								{
                                	int Index = ( ( ( DataArray[0] & 0x3F ) + 1 ) & 0x3F );
                                    DataArray[0] = (byte)( ( DataArray[0] & 0x80 ) | Index );
                                    if( ( Index & 0x01 ) != 0 )
                                    {
										DataArray[1] = (byte)( PaletteArray[Index>>1] >> 8 );
									}
									else
									{
										DataArray[1] = (byte)PaletteArray[Index>>1];
									}
								}
								break;
							default:
								break;
						}
                	}
				}

			}
			
		}
		
	}
	
}






