using System;
using GameKnight.Core;

using Framework = Microsoft.Xna.Framework;
using Graphics = Microsoft.Xna.Framework.Graphics;
using Input = Microsoft.Xna.Framework.Input;

namespace GameKnight.XNA
{

	public partial class Application : Framework.Game, IInterface 
	{
		
		private IConsole Console = null;

		private int FPS = 0;
        
		private Framework.GraphicsDeviceManager GraphicsManager = null;
       
		private InputDevice InputDevice = null;
        
		private int LastSecond = 0;

		public IOptions Options 
		{
			get
			{
				return OptionsProperty;
			}
		}
		
		private IOptions OptionsProperty = new Options();

		private Graphics.SpriteBatch SpriteBatch = null;
       
		private Graphics.Texture2D Texture = null;

		public ITimer Timer 
		{
			get
			{
				return TimerProperty;
			}
		}
		
		private Timer TimerProperty = new Timer();

        private Application()
        {
            GraphicsManager = new Framework.GraphicsDeviceManager( this );
        	return;
        }
       
        protected override void Draw( Framework.GameTime GameTime )
        {
            SpriteBatch.Begin( Graphics.SpriteBlendMode.None );
            SpriteBatch.Draw( Texture, Framework.Vector2.Zero, Graphics.Color.White );
            SpriteBatch.End();
            base.Draw( GameTime );
            return;
        }
        
        public void Error( object Sender, string Message )
        {
        	return;
        }
        
        protected override void Initialize()
        {
            Window.AllowUserResizing = false;
            base.Initialize();
            GraphicsManager.PreferredBackBufferHeight = 144;
            GraphicsManager.PreferredBackBufferWidth = 160;
            GraphicsManager.PreferMultiSampling = false;
            GraphicsManager.SynchronizeWithVerticalRetrace = false;
            GraphicsManager.ApplyChanges();
           	IsFixedTimeStep = false;
            return;
        }

        protected override void LoadContent()
        {
            SpriteBatch = new Graphics.SpriteBatch( GraphicsDevice );
           	Texture = new Graphics.Texture2D( GraphicsDevice, 256, 256, 1, Graphics.TextureUsage.Linear, Graphics.SurfaceFormat.Color );
          	Console = new GameBoy.Console( this );
           	Console.Load( new InputStream( "Minesweeper (PD).gb" ) );
            InputDevice = new InputDevice( Console.ControllerArray[0] );
            return;
        }
	
        public void Present( uint[] ColorArray )
		{
			GraphicsDevice.Textures[0] = null;
			Texture.SetData<uint>( ColorArray, 0, ColorArray.Length, Graphics.SetDataOptions.Discard );
			FPS++;
			return;
		}
       	
        protected override void Update( Framework.GameTime GameTime )
        {
       		if( GameTime.TotalGameTime.Seconds != LastSecond )
       		{
       			LastSecond = GameTime.TotalGameTime.Seconds;
       			Window.Title = String.Format( "{0} FPS", FPS );
       			FPS = 0;
       		}	
       		InputDevice.Update();
       		TimerProperty.Update( GameTime );
       		Console.Update();
            base.Update( GameTime );
            return;
        }
       	
		[STAThread]
		private static void Main( string[] ArgumentArray )
		{
			using( Application Application = new Application() )
            {
                Application.Run();
            }
			return;
		}

	}

}
