using System;
using Collections = System.Collections.Generic;
using GameKnight.Core;

using Framework = Microsoft.Xna.Framework;
using Input = Microsoft.Xna.Framework.Input;

namespace GameKnight.XNA
{

	public sealed class InputDevice
	{
		
		private readonly IController Controller = null;
		
		private static readonly Collections.Dictionary<IControl,string> ControlMappingList = new Collections.Dictionary<IControl,string>();
		
		private static readonly Collections.Dictionary<string,Input.Keys> KeyMappingList = new Collections.Dictionary<string,Input.Keys>();
		
		static InputDevice()
		{
			KeyMappingList.Add( "Button A", Input.Keys.NumPad6 );
			KeyMappingList.Add( "Button B", Input.Keys.NumPad4 );
			KeyMappingList.Add( "Select", Input.Keys.Space );
			KeyMappingList.Add( "Start", Input.Keys.Enter );
			KeyMappingList.Add( "Right", Input.Keys.D );
			KeyMappingList.Add( "Left", Input.Keys.A );
			KeyMappingList.Add( "Up", Input.Keys.W );
			KeyMappingList.Add( "Down", Input.Keys.S );
			return;
		}
		
		public InputDevice( IController Controller )
		{
			this.Controller = Controller;
			foreach( IControl CurrentControl in Controller.ControlArray )
			{
				ControlMappingList.Add( CurrentControl, CurrentControl.Name );
			}
			return;
		}
		
		public int GetValue( IControl Control )
		{
			return (int)KeyMappingList[Control.Name];
		}
		
		public void SetValue( IControl Control, int Value )
		{
			KeyMappingList[Control.Name] = (Input.Keys)Value;
			//ConfigFile.WriteInt( Control.Controller.Name, Control.Name, Value );
			return;
		}
		
		public void Update()
		{
			Input.KeyboardState KeyboardState = Input.Keyboard.GetState();
			Collections.IEnumerator<Collections.KeyValuePair<IControl,string>> Enumerator = ControlMappingList.GetEnumerator();
			do
			{
				if( Enumerator.Current.Key != null )
				{
					Enumerator.Current.Key.Controller[Enumerator.Current.Key] = Convert.ToInt32( KeyboardState.IsKeyDown( KeyMappingList[Enumerator.Current.Value] ) );
				}
			}
			while( Enumerator.MoveNext() );
			return;
		}
		
	}
	
}


